#ifndef _CSRMATRIX_H_
#define _CSRMATRIX_H_

// Hilfsprogramme 
#include <iostream>
#include "math.h"
using namespace std;
#include <cstdlib>

void gitter2(int n, int m,int* column_indices,double* entries,
	     int* row_offsets,double* a)
// generiert Laplace 2d zu nxm Gitter auf (0,1) als CSR-Format
// Gewichte: a(1) in x und a(2) in y Richtung, a(3) Massenmatrix
// matrix ist mit hx*hy Multipliziert.
{
  int i,j,l,pos,nnz,ke;
  double hx,hy;
  pos=0;
  nnz=0;
  ke=0;
  hx=1/double(n+1);
  hy=1/double(m+1);
  for (j=0;j<m;j++)
      {
      for (i=0;i<n;i++) 
        {
          l=0; 
      	  if (j>0) 
	    {
	      entries[ke]=-a[1]*hx/hy;
              column_indices[ke++]=pos-n;
              l++; 
            }
           if (i>0) 
	    {
	      entries[ke]=-a[0]*hy/hx;
              column_indices[ke++]=pos-1;
              l++ ;
            } 
          entries[ke]=2*a[0]*hy/hx+2*a[1]*hx/hy+hx*hy*a[2];
          column_indices[ke++]=pos;
          l++; 
           if (i<n-1) 
	    {
	      entries[ke]=-a[0]*hy/hx;
              column_indices[ke++]=pos+1;
              l++; 
            } 
           if (j<m-1) 
	    {
	      entries[ke]=-a[1]*hx/hy;
              column_indices[ke++]=pos+n;
              l++;
            } 
           row_offsets[pos]=nnz;
           nnz=nnz+l;
           pos++;
	}
      } 
  row_offsets[pos]=nnz; 
}

void gitter1(int n, int* column_indices,double* entries,
	     int* row_offsets,double* a)
// generiert Laplace 2d zu n Gitter auf (0,1) als CSR-Format
// Gewichte: a(1) in x Richtung, a(2) Massenmatrix
// matrix ist mit hx Multipliziert.
{
  int i,j,l,pos,nnz,ke;
  double hx;
  pos=0;
  nnz=0;
  ke=0;
  hx=1/double(n+1);
  for (i=0;i<n;i++)
      {
       l=0; 
       if (i>0) 
       {
         entries[ke]=-a[0]/hx;
         column_indices[ke++]=pos-1;
         l++; 
        }
       entries[ke]=2*a[0]/hx+hx*a[1];
       column_indices[ke++]=pos;
       l++; 
       if (i<n-1) 
       {
         entries[ke]=-a[0]/hx;
         column_indices[ke++]=pos+1;
         l++; 
        } 
        row_offsets[pos]=nnz; 
        nnz=nnz+l;
        pos++;
      } 
  row_offsets[pos]=nnz; 
}


void gitter3(int n, int m,int l,int* column_indices,double* entries,
	     int* row_offsets,double* a)
// generiert Laplace 2d zu nxmxl Gitter auf (0,1) als CSR-Format
// Gewichte: a(1) in x und a(2) in y Richtung,a(3) in z-Richtung, 
//  a(4) Massenmatrix
// matrix ist mit hx*hy*hz multipliziert
{
  int i,j,k,s,pos,nnz,ke;
  double hx,hy,hz;
  pos=0;
  nnz=0;
  ke=0;
  hx=1/double(n+1);
  hy=1/double(m+1);
  hz=1/double(l+1);
  for (k=0;k<l;k++)
    {
    for (j=0;j<m;j++) 
      {
      for (i=0;i<n;i++) 
        { 
          s=0; 
   	  if (k>0) 
	    {
	      entries[ke]=-a[2]*hx*hy/hz;
              column_indices[ke++]=pos-n*m;
              s++; 
            }
           if (j>0) 
	    {
	      entries[ke]=-a[1]*hx*hz/hy;
              column_indices[ke++]=pos-n;
              s++ ;
            } 
           if (i>0) 
	    {
	      entries[ke]=-a[0]*hz*hy/hx;
              column_indices[ke++]=pos-1;
              s++ ;
            } 
          entries[ke]=2*a[0]*hz*hy/hx+2*a[1]*hz*hx/hy
            +2*a[2]*hx*hy/hz+hx*hy*hz*a[3];
          column_indices[ke++]=pos;
          s++; 
           if (i<n-1) 
	    {
	      entries[ke]=-a[0]*hy*hz/hx;
              column_indices[ke++]=pos+1;
              s++;
            } 
           if (j<m-1) 
	    {
	      entries[ke]=-a[1]*hz*hx/hy;
              column_indices[ke++]=pos+n;
              s++; 
            } 
           if (k<l-1) 
	    {
	      entries[ke]=-a[2]*hx*hy/hz;
              column_indices[ke++]=pos+n*m;
              s++;
            } 
           
           row_offsets[pos]=nnz;
           nnz=nnz+s;
           pos++;
	}
	}
      } 
  row_offsets[pos]=nnz; 
}



#endif
